class EffectTemplate {
	string m_scriptName;
	string m_displayName;
	TimeType m_timeType;
	EffectAvailability m_availability;
	bool m_isEnabled = true;

	PollEffect@ generatePollEffect() {
		string variationDisplayName = m_displayName;
		int variationDuration = -1;
		string variationData = "";

		// calculate normal duration
		switch (m_timeType) {
			case TimeType::Normal :
				variationDuration = int(Math::Round(Setting_EffectDuration * 1000));
				break;
			case TimeType::Short :
				variationDuration = int(Math::Round(Setting_ShortEffectDuration * 1000));
				break;
			case TimeType::Instant :
				variationDuration = 0;
				break;
		}

		// Override in RMC
		if (g_chaosMode.m_isRMCMode) {
			switch (m_timeType) {
				case TimeType::Normal :
					variationDuration = 30000;
					break;
				case TimeType::Short :
					variationDuration = 10000;
					break;
				case TimeType::Instant :
					variationDuration = 0;
					break;
			}
		}

		// Special timing and variable effects
		if (m_scriptName == "FakeFinish") {
			variationDuration = 5000;
		} else if (m_scriptName == "FakeIntro") {
			variationDuration = 10000;
		} else if (m_scriptName == "LowControl") {
			uint actionKey = Math::Rand(1,4);
			variationDisplayName = "Action key " + actionKey;
			variationData = tostring(0.2 * actionKey);
		} else if (m_scriptName == "LowAdherence") {
			uint materialType = Math::Rand(0,3);
			switch (materialType) {
				case 0:
					variationDisplayName = "Driving on soap";
					variationData = "0.2";
					break;
				case 1:
					variationDisplayName = "My tires are dead";
					variationData = "0.01";
					break;
				case 2:
					variationDisplayName = "Some weird steering idk";
					variationData = "0.75";
					break;
			}
		} else if (m_scriptName == "LowGravity") {
			uint gravityType = Math::Rand(0,3);
			switch (gravityType) {
				case 0:
					variationDisplayName = "Spaaace";
					variationData = "0.1";
					break;
				case 1:
					variationDisplayName = "Moon gravity";
					variationData = "0.4";
					break;
				case 2:
					variationDisplayName = "Mars gravity";
					variationData = "0.7";
					break;
			}
		} else if (m_scriptName == "FakeGamecrash") {
			variationDuration = 3000;
		}

		return PollEffect(m_scriptName, variationDisplayName, variationDuration, variationData);
	}

	EffectTemplate(const string &in scriptName, const string &in displayName, int timeType, int availability) {
		m_scriptName = scriptName;
		m_displayName = displayName;
		m_timeType = TimeType(timeType);
		m_availability = EffectAvailability(availability);
	}
}

enum TimeType {
	Normal = 1,
	Short = 2,
	Instant = 0,
	Special = 3
}

enum EffectAvailability {
	SingleplayerOnly = 1,
	MultiplayerOnly = 2,
	SingleAndMultiplayer = 3,
	None = 0
}