class TwitchPlaysEffect : ActiveEffect {
	uint64 m_nextBufferSend;
	uint m_totalVotes = 0;
	int m_leftVotes = 0;
	int m_rightVotes = 0;
	int m_brakeVotes = 0;

	void addInput(TwitchPlays::InputType input) {
		switch (input) {
			case TwitchPlays::InputType::Left : {
				m_leftVotes++;
				break;
			}
			case TwitchPlays::InputType::Right: {
				m_rightVotes++;
				break;
			}
			case TwitchPlays::InputType::Brake: {
				m_brakeVotes++;
				break;
			}
		}
		m_totalVotes++;
	}

	string[] generateEventArray() {
		string[] result = {};
		float steer = m_rightVotes - m_leftVotes;
		float brake = m_brakeVotes;
		if (m_totalVotes > 0) {
			steer /= m_totalVotes;
			brake /= m_totalVotes;
		}
		result.InsertLast(tostring(steer));
		result.InsertLast(tostring(brake));
		return result;
	}

	void resetBuffer() {
		m_totalVotes = 0;
		m_leftVotes = 0;
		m_rightVotes = 0;
		m_brakeVotes = 0;
	}

	void ApplyLoop() override {
		if (Time::Now > m_nextBufferSend) {
			if (m_totalVotes > 0) {
				SendCustomEvent("ChaosMode.PlayerInput", generateEventArray());
				resetBuffer();
				m_nextBufferSend = Time::Now + 2000;
			}
		}
	}

	TwitchPlaysEffect(const string &in scriptName, uint duration) {
		super(scriptName, duration);
		m_nextBufferSend = Time::Now + 2000;
	}
}

namespace TwitchPlays {
	enum InputType {
		Left = 0,
		Right = 1,
		Brake = 2
	}
}

class FakeGamecrashEffect : ActiveEffect {
	float m_prevSoundVolume;
	float m_prevMusicVolume;

	bool ApplyStart() override  {
		m_prevSoundVolume = GetApp().SystemOverlay.MasterSoundVolume;
		m_prevMusicVolume = GetApp().SystemOverlay.MasterMusicVolume;
		GetApp().Viewport.Config.FreezeViewportRun = true;
		GetApp().SystemOverlay.MasterSoundVolume = -100;
		GetApp().SystemOverlay.MasterMusicVolume = -100;
		return true;
	}

	bool ApplyEnd() override {
		GetApp().Viewport.Config.FreezeViewportRun = false;
		GetApp().SystemOverlay.MasterSoundVolume = m_prevSoundVolume;
		GetApp().SystemOverlay.MasterMusicVolume = m_prevMusicVolume;
		GetApp().SystemOverlay.MinimizeApp();
		return true;
	}

	FakeGamecrashEffect(const string &in scriptName, uint duration) {
		super(scriptName, duration);
	}
}

class FakeFinishEffect : ActiveEffect {
	bool ApplyStart() override {
		return SetClientUISequence(CGamePlaygroundUIConfig::EUISequence::Finish);
	}

	bool ApplyEnd() override {
		return SetClientUISequence(CGamePlaygroundUIConfig::EUISequence::None);
	}
	
	FakeFinishEffect(const string &in scriptName, uint duration) {
		super(scriptName, duration);
	}
}

class FakeIntroEffect : ActiveEffect {
	bool ApplyStart() override {
		return SetClientUISequence(CGamePlaygroundUIConfig::EUISequence::RollingBackgroundIntro);
	}

	bool ApplyEnd() override {
		return SetClientUISequence(CGamePlaygroundUIConfig::EUISequence::None);
	}

	FakeIntroEffect(const string &in scriptName, uint duration) {
		super(scriptName, duration);
	}
}

class ChatAsBigMessageEffect : ActiveEffect {
	string m_message = "";
	uint64 m_lastSend;

	void ApplyLoop() override {
		if (Time::Now > m_lastSend + 250) {
			if (SetClientUIBigMessage(m_message)) m_lastSend = Time::Now;
		}
	}

	bool ApplyEnd() override {
		return SetClientUIBigMessage("");
	}

	ChatAsBigMessageEffect(const string &in scriptName, uint duration) {
		super(scriptName, duration);
	}
}