// classes
class Map {
	string m_displayName;
	string m_path;

	Map(const string &in displayName, const string &in path) {
		m_displayName = displayName;
		m_path = path;
	}
}

class Folder {
	string m_name;
	array<Map@> m_maps;
	array<Folder@> m_folders;
	bool m_selected;
	
	Folder(const string &in name) {
		m_name = name;
	}
}

// Task
void RefreshIndexedMaps() {
	// Reset map selection
	g_chaosMode.m_selectedMaps = {};

	// List all maps
	string userGameFolderPath = IO::FromUserGameFolder("");
	string[] foundFiles = IO::IndexFolder(userGameFolderPath + "Maps", true); // Indexes all files recursively in Maps folder
	@g_chaosMode.m_indexedMapRoot = Folder("Maps");

	// Check each found file
	for (uint i = 0; i < foundFiles.Length; i++) {
		if (!foundFiles[i].EndsWith(".Map.Gbx")) continue;

		// @TODO validation testing here?

		// Make path relative to User folder
		foundFiles[i] = foundFiles[i].Replace(userGameFolderPath, "");

		array<string> subFolders = foundFiles[i].Split("/");
		Folder@ currentFolder = g_chaosMode.m_indexedMapRoot;
		if (subFolders[0] != "Maps") throw("subFolders[0] == " + subFolders[0]);

		// Build subfolder tree
		for (uint j = 1; j < subFolders.Length; j++) {
			if (j == subFolders.Length - 1) {
				// Done with tree building, map is indexed by filename after subfolder
				currentFolder.m_maps.InsertLast(Map(subFolders[j], "file://" + foundFiles[i]));
			} else {
				// Add folder to tree if nonexistant
				bool folderFound = false;
				for (uint k = 0; k < currentFolder.m_folders.Length; k++) {
					if (currentFolder.m_folders[k].m_name == subFolders[j]) {
						@currentFolder = currentFolder.m_folders[k];
						folderFound = true;
						break;
					}
				}

				if (!folderFound) {
					Folder@ newFolder = Folder(subFolders[j]);
					currentFolder.m_folders.InsertLast(newFolder);
					@currentFolder = newFolder;
				}
			}
		}
	}
}

// Helpers
void UpdateFolderSelection(Folder@ folder, bool selected) {
	folder.m_selected = selected;
	for (uint i = 0; i < folder.m_folders.Length; i++) {
		UpdateFolderSelection(folder.m_folders[i], selected);
	}
	for (uint i = 0; i < folder.m_maps.Length; i++) {
		UpdateMapSelection(folder.m_maps[i], selected);
	}
}

void UpdateMapSelection(Map@ map, bool selected) {
	int mapIndex = FindMapSelectionIndex(map);
	if (selected) {
		if (mapIndex < 0) {
			g_chaosMode.m_selectedMaps.InsertLast(map);
		}
	} else if (mapIndex >= 0) {
		g_chaosMode.m_selectedMaps.RemoveAt(mapIndex);
	}
}

int FindMapSelectionIndex(Map@ map) {
	for (uint i = 0; i < g_chaosMode.m_selectedMaps.Length; i++) {
		if (map.m_path == g_chaosMode.m_selectedMaps[i].m_path) return i;
	}
	return -1;
}