class Poll {
	int m_timeLeft;
	int m_duration;
	uint64 m_lastNow;
	array<PollOption@> m_options;
	string[] m_voters;
	uint m_totalVotes;

	uint LeadingIndex() {
		if (Setting_ProportionalVoting) {
			float rand = Math::Rand(0.0,1.0);
			uint maxVotes = m_options[0].m_votes;
			float runningOffset = 0.0;
			for (uint i = 1; i < m_options.Length; i++) {
				if (runningOffset + float(m_options[i].m_votes) / float(maxVotes) > rand) {
					return i;
				} else {
					runningOffset += float(m_options[i].m_votes) / float(maxVotes);
				}
			}
		}
		uint maxIndex = 0;
		uint maxVotes = m_options[0].m_votes;
		for (uint i = 1; i < m_options.Length; i++) {
			if (m_options[i].m_votes > maxVotes || (m_options[i].m_votes == maxVotes && Math::Rand(0.0,1.0) > 0.5)) {
				maxVotes = m_options[i].m_votes;
				maxIndex = i;
			}
		}
		return maxIndex;
	}

	// Generate new poll
	Poll(int duration) {
		if (duration == 0) throw("Poll duration 0 in constructor!");
		m_duration = duration;
		m_timeLeft = m_duration;
		array<EffectTemplate@> remainingEffects = g_chaosMode.m_effectTemplates;

		// Should the selectors be increased? (to avoid delay-dependent wrong guesses)
		uint baseOptionValue = g_chaosMode.m_nextPollAlternativeSelectors ? Setting_PollOptions : 0;
		g_chaosMode.m_nextPollAlternativeSelectors = Setting_AlternatePollOptions ? !g_chaosMode.m_nextPollAlternativeSelectors : false;

		// Remove already existing special effects to be available
		for (uint i = 0; i < remainingEffects.Length; i++) {
			if (
				(g_chaosMode.m_gameMode == GameMode::Multi && (remainingEffects[i].m_availability & EffectAvailability::MultiplayerOnly == 0)) ||
				(g_chaosMode.m_gameMode != GameMode::Multi && (remainingEffects[i].m_availability & EffectAvailability::SingleplayerOnly == 0)) ||
				!remainingEffects[i].m_isEnabled
			) {
				remainingEffects.RemoveAt(i);
				i--;
				continue;
			}
			for (uint j = 0; j < g_chaosMode.m_activeEffects.Length; j++) {
				if (remainingEffects[i].m_scriptName == g_chaosMode.m_activeEffects[j].m_scriptName) {
					remainingEffects.RemoveAt(i);
					i--;
					break;
				}
			}
		}

		// Generate poll options
		for (uint i = 0; i < uint(Setting_PollOptions); i++) {
			uint nextIndex = Math::Rand(0, remainingEffects.Length);
			PollEffect@ newEffect = remainingEffects[nextIndex].generatePollEffect();
			if (remainingEffects.Length > 2) remainingEffects.RemoveAt(nextIndex);
			m_options.InsertLast(PollOption(newEffect, 1 + baseOptionValue + i));
		}
		m_lastNow = Time::Now;
	}

	// this apparently doesn't work!
	/*
	Poll() {
		Poll(int(Math::Round(Setting_PollDuration * 1000.0)));
	}
	*/
}

class PollOption {
	PollEffect@ m_effect;
	uint m_votes;
	uint m_selector;

	PollOption(PollEffect@ effect, uint selector) {
		@m_effect = effect;
		m_selector = selector;
	}
}

void PollLoop() {
	while (true) {
		yield();
		
		if (g_chaosMode.m_currentPoll is null) continue;

		auto playground = GetApp().CurrentPlayground;
		if (
			playground is null || 
			playground.GameTerminals.Length != 1 || 
			cast<CSmPlayer>(playground.GameTerminals[0].ControlledPlayer) is null ||
			!cast<CSmScriptPlayer>(cast<CSmPlayer>(playground.GameTerminals[0].ControlledPlayer).ScriptAPI).IsEntityStateAvailable
		) {
			g_chaosMode.m_currentPoll.m_lastNow = Time::Now;
			continue;
		}

		if (g_chaosMode.m_isRMCMode && g_chaosMode.m_wantsRMCPause) {
			g_chaosMode.m_currentPoll.m_lastNow = Time::Now;
			continue;
		}

		g_chaosMode.m_currentPoll.m_timeLeft -= Time::Now - g_chaosMode.m_currentPoll.m_lastNow;
		g_chaosMode.m_currentPoll.m_lastNow = Time::Now;

		if (g_chaosMode.m_currentPoll.m_timeLeft < 0) {
			g_chaosMode.m_currentPoll.m_options[g_chaosMode.m_currentPoll.LeadingIndex()].m_effect.Start();
			int nextPollDuration = int(Math::Round(Setting_PollDuration * 1000.0));
			if (g_chaosMode.m_isRMCMode) nextPollDuration = 25000;
			@g_chaosMode.m_currentPoll = Poll(nextPollDuration);
		}
	}
}