void RMCLoop() {
	bool[] previousEffectSettings;

	// Yield so all effects are definitely loaded
	yield();

	while (true) {
		if (g_chaosMode.m_wantsRMCMode) {
			if (!g_chaosMode.m_isRMCMode || !IsInLockedRun()) {
				// End current run
				@g_chaosMode.m_currentPoll = null;

				// Setup randomizer to only include certain effects
				previousEffectSettings = array<bool>();
				for (uint i = 0; i < g_chaosMode.m_effectTemplates.Length; i++) {
					previousEffectSettings.InsertLast(g_chaosMode.m_effectTemplates[i].m_isEnabled);
				}
				ApplyRMCEffectSettings();

				// Start randomizer and lock menu
				g_chaosMode.m_gameMode = GameMode::Solo;
				g_chaosMode.m_ui.m_tabState = TabState::RunningLocked;
				g_chaosMode.m_isRMCMode = true;
				@g_chaosMode.m_currentPoll = Poll(25000);
			}
		} else {
			if (g_chaosMode.m_isRMCMode) {
				for (uint i = 0; i < g_chaosMode.m_effectTemplates.Length; i++) {
					g_chaosMode.m_effectTemplates[i].m_isEnabled = previousEffectSettings[i];
				}

				// Stop run and enter main menu
				g_chaosMode.m_ui.m_tabState = TabState::MainMenu;
				g_chaosMode.m_isRMCMode = false;
				@g_chaosMode.m_currentPoll = null;
			}
		}

		yield();
	}
}

void ApplyRMCEffectSettings() {
	const string[] forcedEffects = {
		"LowAdherence",
		"LowControl",
		"LowAcceleration",
		"LowGravity",
		"Reset",
		"FreeWheeling",
		"ForceAccelerate",
		"NoBrakes",
		"NoSteering",
		"BoostUp",
		"BoostDown",
		"Boost2Up",
		"Boost2Down",
		"SlowMotion",
		"Fragile",
		"CruiseControl",
		"NoBrakeForceAccelerate",
		"Respawn",
		"TeleportToRandomCP",
		"Turbo",
		"FakeGamecrash",
		"Tornado"
	};
	for (uint i = 0; i < g_chaosMode.m_effectTemplates.Length; i++) {
		g_chaosMode.m_effectTemplates[i].m_isEnabled = (forcedEffects.Find(g_chaosMode.m_effectTemplates[i].m_scriptName) >= 0);
	}
}

bool IsInLockedRun() {
	return g_chaosMode.m_ui.m_tabState == TabState::RunningLocked && g_chaosMode.m_currentPoll !is null;
}

namespace ChaosMode {
	bool IsInRMCMode() {
		return g_chaosMode.m_isRMCMode;
	}
	
	void SetRMCMode(bool isEnabled) {
		g_chaosMode.m_wantsRMCMode = isEnabled;
	}

	void SetRMCPaused(bool isPaused) {
		g_chaosMode.m_wantsRMCPause = isPaused;
	}
}