class UI {
	TabState m_tabState;

	void Render() {
		switch (m_tabState) {
			case TabState::MainMenu : 
				RenderMainMenu();
				break;
			case TabState::MapSelection : 
				RenderMapSelection();
				break;
			case TabState::MapOrder : 
				RenderMapOrder();
				break;
			case TabState::ModeSettings : 
				RenderModeSettings();
				break;
			case TabState::FreeplaySettings : 
				RenderFreeplaySettings();
				break;
			case TabState::Running : 
				RenderInGame();
				break;
			case TabState::RunningLocked : 
				RenderInLockedRun();
				break;
			default: 
				UI::Text("\\$faaOops! I don't know what to render here.");
		}
	}

	// Render "main menu"
	void RenderMainMenu() {
		if (UI::Button("Freeplay")) {
			g_chaosMode.m_gameMode = GameMode::Freeplay;
			startnew(RefreshIndexedMaps);
			m_tabState = TabState::MapSelection;
		}
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5) - 25));
		CenterLabel("Trackmania Chaos Mode " + Meta::ExecutingPlugin().Version);
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5)));
		if (CenterButton(Icons::Play + "  Start a new run")) {
			g_chaosMode.m_gameMode = GameMode::Solo;
			startnew(RefreshIndexedMaps);
			m_tabState = TabState::MapSelection;
		}
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, Math::Max((UI::GetWindowSize().y * 0.5) + 30, UI::GetWindowSize().y * 0.8)));
		if (CenterButton(Icons::Cogs + "  Start multiplayer randomizer")) {
			if (cast<CGameManiaPlanet>(GetApp()).ManiaPlanetScriptAPI.ActiveContext == CGameManiaPlanetScriptAPI::EContext::Multi) {
				g_chaosMode.m_gameMode = GameMode::Multi;
				m_tabState = TabState::ModeSettings;
			}
		}
		if (UI::IsItemHovered()) {
			UI::BeginTooltip();
			UI::Text("If you have a local / dedicated server setup with the script already,\nenable this to try adding effects from your client!\nYou need to be in the server to activate this.");
			UI::EndTooltip();
		}
	}

	// Render map selection
	void RenderMapSelection() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("MapSelection", tabSize);
		BoldLabel("\\$faaSelecting incompatible maps may force you to restart the game.");
		if (g_chaosMode.m_indexedMapRoot !is null) {
			UI::PushStyleColor(UI::Col::FrameBg, vec4(0.5,0.5,0.5,0.1));
			UI::PushStyleColor(UI::Col::FrameBgHovered, vec4(1.0,1.0,1.0,0.1));
			UI::PushStyleColor(UI::Col::FrameBgActive, vec4(0.75,0.75,0.75,0.1));
			UI::PushStyleColor(UI::Col::CheckMark, vec4(0.357,0.888,0.922, 1.0));
			RenderFolder(g_chaosMode.m_indexedMapRoot);
			UI::PopStyleColor(4);
		}
		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();
		vec2 pos_orig = UI::GetCursorPos();
		if (UI::Button(Icons::AngleDoubleLeft + "  Menu")) {
			m_tabState = TabState::MainMenu;
		}

		UI::SetCursorPos(vec2(UI::GetWindowSize().x * 0.5 - Draw::MeasureString(Icons::Redo + "       maps selected" + g_chaosMode.m_selectedMaps.Length).x * 0.5, pos_orig.y));
		if (UI::Button(Icons::Redo)) {
			startnew(RefreshIndexedMaps);
		}
		UI::SameLine();
		UI::Text(g_chaosMode.m_selectedMaps.Length + " maps selected");


		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString("Map order        " + Icons::AngleDoubleRight).x, pos_orig.y));
		UI::BeginDisabled(g_chaosMode.m_selectedMaps.Length < 1);
		if (UI::Button("Map order  " + Icons::AngleDoubleRight)) {
			m_tabState = TabState::MapOrder;
		}
		UI::EndDisabled();
	}

	// Render map order
	void RenderMapOrder() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("MapOrder", tabSize);
		for (uint i = 0; i < g_chaosMode.m_selectedMaps.Length; i++) {
			UI::PushID(g_chaosMode.m_selectedMaps[i]);
			if (UI::Button(Icons::AngleUp)) {
				Map@ selectedMap = g_chaosMode.m_selectedMaps[i];
				g_chaosMode.m_selectedMaps.RemoveAt(i);
				g_chaosMode.m_selectedMaps.InsertAt(Math::Max(0, i-1), selectedMap);
			}
			UI::SameLine();
			if (UI::Button(Icons::AngleDown)) {
				Map@ selectedMap = g_chaosMode.m_selectedMaps[i];
				g_chaosMode.m_selectedMaps.RemoveAt(i);
				g_chaosMode.m_selectedMaps.InsertAt(Math::Min(g_chaosMode.m_selectedMaps.Length, i+1), selectedMap);
			}
			UI::SameLine();
			UI::Text(g_chaosMode.m_selectedMaps[i].m_displayName);
			UI::PopID();
		}
		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();

		vec2 pos_orig = UI::GetCursorPos();
		if (UI::Button(Icons::AngleDoubleLeft + "  Map selection")) {
			m_tabState = TabState::MapSelection;
		}

		string nextPageString = "???";
		if (g_chaosMode.m_gameMode == GameMode::Freeplay) {
			nextPageString = "Freeplay Settings";
		} else {
			nextPageString = "Mode Settings";
		}
		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString(nextPageString + "        " + Icons::AngleDoubleRight).x, pos_orig.y));
		if (UI::Button(nextPageString + "  " + Icons::AngleDoubleRight)) {
			if (g_chaosMode.m_gameMode == GameMode::Freeplay) {
				m_tabState = TabState::FreeplaySettings;
			} else {
				m_tabState = TabState::ModeSettings;
			}
		}
	}

	// Render mode settings
	void RenderModeSettings() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("ModeSettings", tabSize);

		// List effects
		UI::ListClipper Clipper(g_chaosMode.m_effectTemplates.Length);
		while (Clipper.Step()) {
			for (int i = Clipper.DisplayStart; i < Clipper.DisplayEnd; i++) {
				UI::PushID(g_chaosMode.m_effectTemplates[i].m_scriptName);
				g_chaosMode.m_effectTemplates[i].m_isEnabled = UI::Checkbox(
					g_chaosMode.m_effectTemplates[i].m_displayName + " \\$aaa " + tostring(g_chaosMode.m_effectTemplates[i].m_timeType) + "  " + 
					((g_chaosMode.m_effectTemplates[i].m_availability & EffectAvailability::SingleplayerOnly == EffectAvailability::SingleplayerOnly) ? "\\$afa" : "\\$faa") + Icons::User + 
					((g_chaosMode.m_effectTemplates[i].m_availability & EffectAvailability::MultiplayerOnly == EffectAvailability::MultiplayerOnly) ? "\\$afa" : "\\$faa") + Icons::Users
				, g_chaosMode.m_effectTemplates[i].m_isEnabled);
				UI::PopID();
			}
		}

		RenderRefreshButton();

		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();

		vec2 pos_orig = UI::GetCursorPos();

		string backPageLabel = "???";
		TabState backTabState = TabState::MainMenu;
		switch (g_chaosMode.m_gameMode) {
			case GameMode::Multi:
				backPageLabel = "  Menu";
				backTabState = TabState::MainMenu;
				break;
			case GameMode::Solo:
				backPageLabel = "  Map order";
				backTabState = TabState::MapOrder;
				break;
		}
		if (UI::Button(Icons::AngleDoubleLeft + backPageLabel)) {
			m_tabState = backTabState;
		}

		UI::BeginDisabled(
			(g_chaosMode.m_gameMode == GameMode::Solo && (!cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady || GetSuitableEffectCount() < 2))
		);
		
		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString("Start        " + Icons::AngleDoubleRight).x, pos_orig.y));
		if (UI::Button("Start  " + Icons::AngleDoubleRight)) {
			switch (g_chaosMode.m_gameMode) {
				case GameMode::Multi:
					StartMultiRun();
					break;
				case GameMode::Solo:
					StartLocalRun();
					break;
			}
		}

		if (g_chaosMode.m_gameMode == GameMode::Solo) {
			if (UI::IsItemHovered() && !cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady) {
				UI::BeginTooltip();
				UI::Text("You have to be in the main menu to start a run!");
				UI::EndTooltip();
			}
		}
		UI::EndDisabled();
	}

	// Select effect bindings for freeplay
	void RenderFreeplaySettings() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("MapOrder", tabSize);

		// List bindings
		int effectScheduledForRemoval = -1;
		UI::ListClipper Clipper(g_chaosMode.m_freeplayBindings.Length);
		while (Clipper.Step()) {
			for (int i = Clipper.DisplayStart; i < Clipper.DisplayEnd; i++) {
				UI::PushID("FreeplayBinding" + i);

				UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.2);
				UI::Text(tostring(g_chaosMode.m_freeplayBindings[i].m_binding));
				UI::SameLine();
				UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.6);
				if (UI::BeginCombo("", g_chaosMode.m_freeplayBindings[i].GetBindingEffect() is null ? "" : g_chaosMode.m_freeplayBindings[i].GetBindingEffect().m_displayName)) {
					for (uint j = 0; j < g_chaosMode.m_effectTemplates.Length; j++) {
						if (UI::Selectable(
							g_chaosMode.m_effectTemplates[j].m_displayName + " \\$aaa " + tostring(g_chaosMode.m_effectTemplates[j].m_timeType) + "  " + 
							((g_chaosMode.m_effectTemplates[j].m_availability & EffectAvailability::SingleplayerOnly == EffectAvailability::SingleplayerOnly) ? "\\$afa" : "\\$faa") + Icons::User + 
							((g_chaosMode.m_effectTemplates[j].m_availability & EffectAvailability::MultiplayerOnly == EffectAvailability::MultiplayerOnly) ? "\\$afa" : "\\$faa") + Icons::Users,
							g_chaosMode.m_freeplayBindings[i].GetBindingEffect().m_scriptName == g_chaosMode.m_effectTemplates[j].m_scriptName
						)) {
							g_chaosMode.m_freeplayBindings[i].m_effectScriptName = g_chaosMode.m_effectTemplates[j].m_scriptName;
						}
					}
					UI::EndCombo();
				}
				UI::SameLine();
				UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.2);
				UI::PushStyleColor(UI::Col::Button, vec4(0.647,0.176,0.157, 1.0));
				UI::PushStyleColor(UI::Col::ButtonHovered, vec4(0.98,0.275,0.251, 1.0));
				UI::PushStyleColor(UI::Col::ButtonActive, vec4(0.98,0.027,0.059, 1.0));
				if (UI::Button(Icons::Trash)) {
					effectScheduledForRemoval = i;
				}
				UI::PopStyleColor(3);
				UI::PopID();
			}
		}

		// quick workaround cause removing an item in a listclipper referenced list at the time the listclipper is active is bad
		if (effectScheduledForRemoval >= 0) {
			g_chaosMode.m_freeplayBindings.RemoveAt(effectScheduledForRemoval);
			effectScheduledForRemoval = -1;
		}

		if (g_chaosMode.m_freeplayBindings.Length > 0) UI::Separator();

		UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.2);
		if (UI::Button(g_chaosMode.m_bindingModeActive ? "Recording..." : g_chaosMode.m_newBinding.m_binding == 0 ? "Click to set keybind" : tostring(g_chaosMode.m_newBinding.m_binding))) {
			g_chaosMode.m_bindingModeActive = !g_chaosMode.m_bindingModeActive;
		}
		UI::SameLine();
		UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.6);
		if (UI::BeginCombo("##FreeplayBindingCombo", g_chaosMode.m_newBinding.GetBindingEffect() is null ? "" : g_chaosMode.m_newBinding.GetBindingEffect().m_displayName)) {
			for (uint i = 0; i < g_chaosMode.m_effectTemplates.Length; i++) {
				if (UI::Selectable(
					g_chaosMode.m_effectTemplates[i].m_displayName + " \\$aaa " + tostring(g_chaosMode.m_effectTemplates[i].m_timeType) + "  " + 
					((g_chaosMode.m_effectTemplates[i].m_availability & EffectAvailability::SingleplayerOnly == EffectAvailability::SingleplayerOnly) ? "\\$afa" : "\\$faa") + Icons::User + 
					((g_chaosMode.m_effectTemplates[i].m_availability & EffectAvailability::MultiplayerOnly == EffectAvailability::MultiplayerOnly) ? "\\$afa" : "\\$faa") + Icons::Users,
					g_chaosMode.m_newBinding.GetBindingEffect() !is null && g_chaosMode.m_newBinding.m_effectScriptName == g_chaosMode.m_effectTemplates[i].m_scriptName
				)) {
					g_chaosMode.m_newBinding.m_effectScriptName = g_chaosMode.m_effectTemplates[i].m_scriptName;
				}
			}
			UI::EndCombo();
		}
		UI::SameLine();
		UI::SetNextItemWidth(UI::GetWindowContentRegionWidth() * 0.2);
		UI::PushStyleColor(UI::Col::Button, vec4(0.157,0.647,0.176, 1.0));
		UI::PushStyleColor(UI::Col::ButtonHovered, vec4(0.251,0.98,0.275, 1.0));
		UI::PushStyleColor(UI::Col::ButtonActive, vec4(0.059,0.98,0.027, 1.0));
		if (UI::Button("+")) {
			g_chaosMode.m_freeplayBindings.InsertLast(FreeplayBinding(g_chaosMode.m_newBinding.m_binding, g_chaosMode.m_newBinding.m_effectScriptName));
			g_chaosMode.m_newBinding = FreeplayBinding();
		}
		UI::PopStyleColor(3);

		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();

		vec2 pos_orig = UI::GetCursorPos();

		if (UI::Button(Icons::AngleDoubleLeft + "  Map order")) {
			m_tabState = TabState::MapOrder;
		}

		UI::BeginDisabled(!cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady);

		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString("Start        " + Icons::AngleDoubleRight).x, pos_orig.y));
		if (UI::Button("Start  " + Icons::AngleDoubleRight)) {
			StartFreeplayRun();
		}

		if (UI::IsItemHovered() && !cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady) {
			UI::BeginTooltip();
			UI::Text("You have to be in the main menu to start a run!");
			UI::EndTooltip();
		}

		UI::EndDisabled();
	}

	bool RenderBinding(VirtualKey key, EffectTemplate@ template, bool bindingExists) {
		UI::Text(tostring(key));
		UI::SameLine();
		UI::Text(template is null ? "" : template.m_displayName);
		return false;
	}

	// Render while runnning poll
	void RenderInGame() {
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5) - 25));
		CenterLabel("Run in progress...");
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5)));
		UI::PushStyleColor(UI::Col::Button, vec4(0.647,0.176,0.157, 1.0));
		UI::PushStyleColor(UI::Col::ButtonHovered, vec4(0.98,0.275,0.251, 1.0));
		UI::PushStyleColor(UI::Col::ButtonActive, vec4(0.98,0.027,0.059, 1.0));
		if (CenterButton(Icons::PowerOff + "  Stop run")) {
			@g_chaosMode.m_currentPoll = null;
			g_chaosMode.m_freeplayActive = false;
			m_tabState = TabState::MainMenu;
		}
		UI::PopStyleColor(3);
	}

	void RenderInLockedRun() {
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5) - 25));
		CenterLabel("Externally controlled run in progress!\nGenerating events until the external plugin\nstops the effect generation.");
	}
	
	// File selection

	void RenderFolder(Folder@ folder) {
		UI::PushID(folder.m_name);
		if (UI::Checkbox("\\$z", folder.m_selected) != folder.m_selected) {
			UpdateFolderSelection(folder, !folder.m_selected);
		}
		UI::SameLine();
		if (UI::TreeNode("\\$ff7" + Icons::FolderO + " \\$fff" + folder.m_name)) {
			for (uint i = 0; i < folder.m_folders.Length; i++) {
				UI::TreeAdvanceToLabelPos();
				RenderFolder(folder.m_folders[i]);
			}
			for (uint i = 0; i < folder.m_maps.Length; i++) {
				UI::TreeAdvanceToLabelPos();
				RenderMap(folder.m_maps[i]);
			}
			UI::TreePop();
		}
		UI::PopID();
	}

	void RenderMap(Map@ map) {
		bool mapSelected = FindMapSelectionIndex(map) >= 0;
		if (UI::Checkbox("\\$5f5" + Icons::FileO + " \\$fff" + map.m_displayName, mapSelected) != mapSelected) {
			UpdateMapSelection(map, !mapSelected);
		}
	}

	// General helpers
	void CenterLabel(const string &in Label) {
		vec2 pos_orig = UI::GetCursorPos();

		vec2 LabelSize = Draw::MeasureString(Label, g_uiFontBold);
		UI::SetCursorPos(vec2((UI::GetWindowSize().x - LabelSize.x) / 2, UI::GetCursorPos().y));
		BoldLabel(Label);

		UI::SetCursorPos(pos_orig);
	}

	void BoldLabel(const string &in Label) {
		UI::PushFont(g_uiFontBold);
		UI::Text(Label);
		UI::PopFont();
	}

	bool CenterButton(const string &in Label, bool bold = false) {
		if (bold) UI::PushFont(g_uiFontBold);
		vec2 pos_orig = UI::GetCursorPos();
		vec2 LabelSize = Draw::MeasureString(Label + "  ", bold ? g_uiFontBold : null);
		UI::SetCursorPos(vec2((UI::GetWindowSize().x - LabelSize.x) / 2, UI::GetCursorPos().y));
		bool result = UI::Button(Label);
		if (bold) UI::PopFont();
		return result;
	}

	void RenderRefreshButton() {
		vec2 pos_orig = UI::GetCursorPos();
		UI::SetCursorPos(vec2(UI::GetWindowSize().x - 70, 0));
		if (UI::Button((g_chaosMode.m_effectTemplates[0].m_isEnabled ? Icons::SquareO : Icons::CheckSquare) + " all")) {
			bool settingToApply = !g_chaosMode.m_effectTemplates[0].m_isEnabled;
			for (uint i = 0; i < g_chaosMode.m_effectTemplates.Length; i++) {
				g_chaosMode.m_effectTemplates[i].m_isEnabled = settingToApply;
			}
		}
		UI::SetCursorPos(pos_orig);
		UI::Dummy(vec2(0,0));
	}
}

enum TabState {
	MainMenu,
	MapSelection,
	MapOrder,
	ModeSettings,
	FreeplaySettings,
	Running,
	RunningLocked
}